#include "DrawingContext.h"

#include "AbstractPainter.h"
#include "PainterFactory.h"

#include "Grid.h"
#include "TriMesh.h"

#include "DrawingSurface.h"
#include "Scene.h"

DrawingContext::DrawingContext(DrawingSurface* surface, FontsManager* fontsManager, DrawingMode requestedDrawingMode)
    : mSurface(surface),
      mDrawingMode(requestedDrawingMode),
      mScene(std::make_unique<Scene>()),
      mFontsManager(fontsManager)
{
    mPainter = PainterFactory::Create(mDrawingMode);
}

std::unique_ptr<DrawingContext> DrawingContext::Create(DrawingSurface* surface, FontsManager* fontsManager, DrawingMode requestedDrawingMode)
{
    return std::make_unique<DrawingContext>(surface, fontsManager, requestedDrawingMode);
}

Scene* DrawingContext::getScene() const
{
    return mScene.get();
}

DrawingSurface* DrawingContext::getSurface() const
{
    return mSurface;
}

FontsManager* DrawingContext::getFontsManager() const
{
    return mFontsManager;
}

void DrawingContext::paint()
{
    mScene->update(mFontsManager, mDrawingMode == DrawingMode::RASTER ? mSurface->getImage() : nullptr);
    mPainter->paint(this);
}
