#pragma once

#include <memory>
#include <vector>

class Scene;
class AbstractPainter;
class DrawingSurface;
class FontsManager;

enum class DrawingMode
{
    RASTER,
    GRAPH
};

class DrawingContext
{
public:
    DrawingContext(DrawingSurface* surface, FontsManager* fontsManager, DrawingMode requestedDrawingMode);

    static std::unique_ptr<DrawingContext> Create(DrawingSurface* surface, FontsManager* fontsManager, DrawingMode requestedDrawingMode);

    Scene* getScene() const;

    DrawingSurface* getSurface() const;

    FontsManager* getFontsManager() const;

    void paint();

private:
    DrawingMode mDrawingMode;
    FontsManager* mFontsManager{nullptr};
    DrawingSurface* mSurface{nullptr};
    std::unique_ptr<Scene> mScene;
    std::unique_ptr<AbstractPainter> mPainter;
};

using DrawingContextPtr = std::unique_ptr<DrawingContext>;
