#include "OpenGlMeshPainter.h"

#include "DrawingContext.h"
#include "DrawingSurface.h"
#include "TriMesh.h"

#include "OpenGlShaderProgram.h"

#ifdef _WIN32
#include <windows.h>
#endif

#define GL_GLEXT_PROTOTYPES
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>

#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include <glm/gtc/type_ptr.hpp>

#include <iostream>

OpenGlMeshPainter::OpenGlMeshPainter()
{

}

void OpenGlMeshPainter::initializeShader()
{
    auto vert_shader_path = std::filesystem::path(__FILE__).parent_path() / "shaders/default.vert";
    auto frag_shader_path = std::filesystem::path(__FILE__).parent_path() / "shaders/default.frag";
    mShaderProgram = std::make_unique<OpenGlShaderProgram>(vert_shader_path, frag_shader_path);
}

void OpenGlMeshPainter::initializeBuffers()
{
    glGenBuffers(1, &mVertexBuffer);

    glGenBuffers(1, &mElementBuffer);

    glGenVertexArrays(1, &mVertexArray);
}

void OpenGlMeshPainter::paint(const std::vector<float>& verts, const std::vector<unsigned>& elements, const std::vector<float>& color)
{
    glBindVertexArray(mVertexArray);

    glBindBuffer(GL_ARRAY_BUFFER, mVertexBuffer);
    glBufferData(GL_ARRAY_BUFFER, verts.size() * sizeof(float), verts.data(), GL_STATIC_DRAW);

    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, mElementBuffer);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, elements.size() * sizeof(unsigned), elements.data(), GL_STATIC_DRAW);

    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), (void*)0);
    glEnableVertexAttribArray(0);

    //glm::mat4 projection = glm::ortho(0.0f, width, 0.0f, height);
    //glUniformMatrix4fv(glGetUniformLocation(mShaderProgram->getHandle(), "projection"), 1, GL_FALSE, glm::value_ptr(projection));

    glUseProgram(mShaderProgram->getHandle());
    glBindVertexArray(mVertexArray);

    int vertexColorLocation = glGetUniformLocation(mShaderProgram->getHandle(), "ourColor");
    glUniform4f(vertexColorLocation, float(color[0]), float(color[1]), float(color[2]), float(color[3]));

    //glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_INT, 0);
    glBindVertexArray(0);
}

void OpenGlMeshPainter::paint(TriMesh* mesh, DrawingContext* context)
{
    if (!mShaderProgram)
    {
        initializeShader();
    }

    if (mVertexArray == 0)
    {
        initializeBuffers();
    }

    auto surface = context->getSurface();
    const auto width = float(surface->getWidth());
    const auto height = float(surface->getHeight());

    auto vertices = mesh->getVerticesFlat<float>();
    for (std::size_t idx = 0; idx<vertices.size(); idx++)
    {
        if (idx % 3 == 0)
        {
            vertices[idx] = 2*vertices[idx]/width - 1.0;
        }
        else if(idx%3 == 1)
        {
            vertices[idx] = 1.0 - 2*vertices[idx]/height;
        }
    }

    const auto indices = mesh->getFaceNodeIds();

    std::vector<float> color(4, 1.0f);
    if (mesh->hasVectorAttribute("Color"))
    {
        auto mesh_color = mesh->getVectorAttribute("Color");
        color = {float(mesh_color[0]), float(mesh_color[1]), float(mesh_color[2]), float(mesh_color[3])};
    }

    paint(vertices, indices, color);
}
