#pragma once

#include "AbstractMesh.h"
#include "Image.h"
#include "DiscretePoint.h"

#include <memory>

class FontsManager;

class AbstractVisualNode
{
public:
    AbstractVisualNode(const DiscretePoint& location)
        : mLocation(location)
    {

    }

    virtual ~AbstractVisualNode() = default;

    AbstractMesh* getMesh() const
    {
        return mMesh.get();
    }

    virtual void update(FontsManager* fontsManager)
    {

    }

    virtual void updateMesh()
    {

    }

    virtual void updateTexture(FontsManager* fontsManager)
    {

    }

    Image<unsigned char>* getImage() const
    {
        return mImage.get();
    }

    const DiscretePoint& getLocation() const
    {
        return mLocation;
    }

    Image<unsigned char>* getTexture() const
    {
        return mTexture.get();
    }

protected:
    DiscretePoint mLocation;
    std::unique_ptr<AbstractMesh> mMesh;
    std::unique_ptr<Image<unsigned char> > mImage;
    std::unique_ptr<Image<unsigned char> > mTexture;
};
