#include "RectangleNode.h"

#include "Rectangle.h"
#include "MeshPrimitives.h"

#include <iostream>

RectangleNode::RectangleNode(const DiscretePoint& loc, unsigned width, unsigned height)
    : GeometryNode(loc),
      mWidth(width),
      mHeight(height)
{

}

std::unique_ptr<RectangleNode> RectangleNode::Create(const DiscretePoint& loc, unsigned width, unsigned height)
{
    return std::make_unique<RectangleNode>(loc, width, height);
}

GeometryNode::Type RectangleNode::getType()
{
    return GeometryNode::Type::Rectangle;
}

unsigned RectangleNode::getWidth() const
{
    return mWidth;
}

unsigned RectangleNode::getHeight() const
{
    return mHeight;
}

void RectangleNode::update(FontsManager* fontsManager)
{
    updateMesh();
}

void RectangleNode::updateMesh()
{
    const auto rect = Rectangle(mLocation, mWidth, mHeight);

    auto mesh = MeshPrimitives::build(rect);
    auto color = getFillColor();

    mesh->addVectorAttribute("Color", color.getAsVectorDouble());
    mMesh = std::move(mesh);
}
