#include "Scene.h"

#include "VisualLayer.h"
#include "GeometryNode.h"
#include "RectangleNode.h"
#include "TextNode.h"

#include "MeshBuilder.h"
#include "TriMesh.h"

#include "Image.h"

void Scene::syncLayers(const std::vector<VisualLayer*>& layers)
{
    mLayers = layers;
}

void Scene::update(FontsManager* fontsManager, Image<unsigned char>* image)
{
    if (image)
    {
        //
    }
    else
    {
        mWorkingMeshs.clear();
        for(auto layer : mLayers)
        {
            if (layer->hasShapeNode())
            {
                auto node = layer->getShapeNode();
                if (layer->getIsDirty())
                {
                    node->update(fontsManager);
                    layer->setIsDirty(false);
                }
                mWorkingMeshs.push_back(dynamic_cast<TriMesh*>(node->getMesh()));
                mTextures.push_back(nullptr);
            }
            else
            {
                auto node = dynamic_cast<TextNode*>(layer->getTextNode());
                if (layer->getIsDirty())
                {
                    node->update(fontsManager);
                    layer->setIsDirty(false);
                }
                mTextData.push_back(node->getTextData());
            }
        }
    }
}

const std::vector<TextData>& Scene::getTextData() const
{
    return mTextData;
}

void Scene::processRectangleNode(RectangleNode* node)
{

}

unsigned Scene::getNumMeshes() const
{
    return mWorkingMeshs.size();
}

TriMesh* Scene::getMesh(std::size_t idx) const
{
    return mWorkingMeshs[idx];
}
