#pragma once

#include "DiscretePoint.h"

#include "AbstractVisualNode.h"
#include "Color.h"
#include "FontItem.h"
#include "TextData.h"

#include <memory>
#include <string>

class TextNode : public AbstractVisualNode
{
public:
    TextNode(const std::string& content, const DiscretePoint& loc);

    ~TextNode();

    static std::unique_ptr<TextNode> Create(const std::string& content, const DiscretePoint& loc);

    const Color& getFillColor() const;
    const Color& getStrokeColor() const;

    std::string getContent() const;
    std::string getFontLabel() const;

    const TextData& getTextData() const;

    void setContent(const std::string& content);
    void setFillColor(const Color& color);
    void setStrokeColor(const Color& color);

    void update(FontsManager* fontsManager) override;

private:
    void updateMesh() override;
    void updateTexture(FontsManager* fontsManager) override;

    TextData mTextData;
};

using TextNodetr = std::unique_ptr<TextNode>;
